/*

.appdv-edit-box {
  visibility:hidden;
  position: absolute;
  top: 0; 
  right: 0;
  padding-top:4px;
  padding-left:4px;
  cursor:  pointer;
  z-index:500;
  font-size: 10px;
  font-family: arial;
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny.png) center right no-repeat floralwhite;
  padding:1px 14px 1px 1px;
  white-space:nowrap;
  color: blue;
  font-weight: normal;
  text-decoration:underline;
  border-style: dotted;
  border-width: 1px;
  border-color: blue;
}

.appdv-field-value:hover, .appdv-field-value:hover .appdv-edit-box {
  visibility:visible;
}


.appdv-edit-box + .save, .appdv-edit-box + .cancel, .editing .appdv-edit-box {
  display:none;
}

.editing .appdv-edit-box + .save, .editing .appdv-edit-box + .cancel {
  display:block;
  visibility:visible;
  position: relative;
  
  background: floralwhite;
  padding-right: 1px;
  float:right;
}

.editing .appdv-edit-box + .save {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_disk.png) center right no-repeat floralwhite;
  padding-right: 15px;
  float:left;
}

*/




/******************  edit-record-toggle  *****************/

.appdv-tt-generated .edit-record-toggle.default {
  padding: 4px;
}

.record-update .appdv-tt-generated .edit-record-toggle .edit {
  display:none!important;
}

.appdv-tt-generated .edit-record-toggle .edit,
.appdv-tt-generated .edit-record-toggle .save,
.appdv-tt-generated .edit-record-toggle .cancel {
  display: none;
  visibility:hidden;
}

.appdv-tt-generated .edit-record-toggle.default .edit,
.appdv-tt-generated .edit-record-toggle.default .save,
.appdv-tt-generated .edit-record-toggle.default .cancel {
  width: 30;
  padding:4px 14px 1px 1px;
  cursor:  pointer;
  font-size: 11px;
  line-height: 12px;
  font-family: arial;
  white-space:nowrap;
  color: blue;
  font-weight: bold;

  border-bottom-width: 1px;
  border-bottom-style: dotted;
  border-color: transparent;
}

.appdv-tt-generated .edit-record-toggle.default .edit:hover,
.appdv-tt-generated .edit-record-toggle.default .save:hover,
.appdv-tt-generated .edit-record-toggle.default .cancel:hover {
  border-color: blue;
}

.appdv-tt-generated .edit-record-toggle .edit {
  display:inline-block;
}

.editing-record .appdv-tt-generated .edit-record-toggle .edit,
.ra-dsapi-deny-update .appdv-tt-generated .edit-record-toggle .edit {
  display:none;
}


.record-over:hover .appdv-tt-generated .edit-record-toggle .edit {
  visibility:visible;
}


.editing-record .appdv-tt-generated .edit-record-toggle .save, 
.editing-record .appdv-tt-generated .edit-record-toggle .cancel {
  visibility:visible;
  display:inline-block;
}

.appdv-tt-generated .edit-record-toggle.default .edit {
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny.png) center right no-repeat;
}

.appdv-tt-generated .edit-record-toggle.default .save {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_disk.png) center right no-repeat;
}

.appdv-tt-generated .edit-record-toggle.default .cancel {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_cross.png) center right no-repeat;
}

/******************  END edit-record-toggle  *****************/



/******************  record-delete  *****************/

.appdv-tt-generated .delete-record, 
.appdv-tt-generated .print-view {
  display:inline-block;
  background:url(../../../assets/rapidapp/misc/static/images/cross.png) center right no-repeat;
  padding:0px 23px 0px 1px;
  width: 30px;
  cursor:  pointer;
  font-size: 12px;
  line-height: 16px;
  font-family: arial;
  white-space:nowrap;
  color: darkgray;
  font-weight: bold;
  border-bottom-width: 1px;
  border-bottom-style: solid;
  border-color: transparent;
  cursor:  pointer;
}

.appdv-tt-generated .print-view {
    background:url(../../../assets/rapidapp/misc/static/images/printer.png) center right no-repeat;
}

.rapidapp-printview .appdv-tt-generated .print-view {
    display: none !important;
}

.appdv-tt-generated .delete-record:hover, 
.appdv-tt-generated .print-view:hover {
  border-bottom-style: dotted;
  border-color: blue;
  color: blue;
}

/*
.editing-record .appdv-tt-generated .delete-record,
.ra-dsapi-deny-destroy .appdv-tt-generated .delete-record {
  display: none !important;
}
*/

/* This rule has been moved into 510-AppDV.js to be dynamically generated for each
   AppDV on an experimental basis. If this works out, many more styles will moved
   to be generated this way, to avoid rules of nested AppDV modules from conflicting
ra-dsapi-deny-destroy .appdv-tt-generated .delete-record, */
.editing-record .appdv-tt-generated .delete-record {
  display: none !important;
}

@media print {
    .appdv-tt-generated .delete-record {
        display: none !important;
    }
}

/******************  END record-delete  *****************/



/*
.appdv-editable-value {
  padding-right: 14px;
}

.appdv-editable-value:hover {
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny.png) center right no-repeat;
  cursor:  pointer;
}

.appdv-editable-value:hover span {
  border-bottom-style: dotted;
  border-bottom-width: 1px;
  border-color: blue;
}
*/


/****************** appdv-edit-field ******************/

.appdv-edit-field .icons {
  width:24px;
}

.appdv-edit-field .icons div {
  width: 12px;
  /*float:left;*/
  cursor:  pointer;
  visibility: hidden;
}

.editing-record .appdv-edit-field .icons div,
.record-update  .appdv-edit-field .icons div,
.non-phantom .no-edit .appdv-edit-field .icons div,
.ra-dsapi-deny-update .non-phantom .appdv-edit-field .icons div {
  visibility: hidden !important;
}


.appdv-edit-field .icons .edit {
  width: 14px;
  visibility: visible;
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny_gray.png) right center no-repeat;
}

.appdv-edit-field .icons .save {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_disk.png) center center no-repeat;
  display:none;
}

.appdv-edit-field .icons .cancel {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_cross.png) center center no-repeat;
  display:none;
}

.appdv-edit-field:hover .icons .edit {
  visibility: visible;
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny.png) right center no-repeat;
}


.appdv-edit-field .data-holder, 
.onepx-transparent-border-bottom {
  border-bottom-style: dotted;
  border-bottom-width: 1px;
  border-color: transparent;
}

/* prevent inheriting padding from parents */
.appdv-edit-field td, 
.appdv-edit-field tr {
  padding: 0px !important;
}

.appdv-edit-field:hover .data-holder {
  border-color: blue;
  cursor:  pointer;
}

.non-phantom .no-edit .appdv-edit-field:hover .data-holder,
.ra-dsapi-deny-update .non-phantom .appdv-edit-field:hover .data-holder {
  border-color: transparent;
  cursor:  default;
}

.editing .appdv-edit-field .icons .save, 
.editing .appdv-edit-field .icons .cancel {
  display:inline-block;
  visibility: visible;
}

.editing .appdv-edit-field .icons .edit {
  visibility: hidden;
  display:none;
}

/****************** END appdv-edit-field ******************/



/****************** appdv-edit-bigfield ******************/

div.appdv-edit-bigfield {
  position:relative;
  min-height: 20px;
}

div.appdv-edit-bigfield .data-wrapper .data-holder {
  padding-right: 50px; /* make room for *edit* */
}

.editing-record .appdv-edit-bigfield .icons div,
.record-update .appdv-edit-bigfield .icons div,
.non-phantom .no-edit .appdv-edit-bigfield .icons div,
.ra-dsapi-deny-update .non-phantom .appdv-edit-bigfield .icons div {
  display:none !important;
}


.appdv-edit-bigfield div.edit,
.appdv-edit-bigfield div.save,
.appdv-edit-bigfield div.cancel {
  display:none;
  position: absolute;
  top: 0; 
  right: 0;
  padding-top:4px;
  padding-left:4px;
  cursor:  pointer;
  z-index:500;
  font-size: 10px;
  line-height: 11px;
  font-family: arial;
  padding:2px 16px 2px 2px;
  margin-right:1px;
  white-space:nowrap;
  color: blue;
  font-weight: normal;
  border-style: dotted;
  border-width: 1px;
  border-color: blue;
}



.appdv-edit-bigfield .icons .edit {
  display:block;
  color:transparent;
  border-color: transparent;
  background:url(../../../assets/rapidapp/misc/static/images/bullet_pencil_gray.png) center right no-repeat;
}

.appdv-edit-bigfield:hover .icons .edit {
  color:grey;
}

.appdv-edit-bigfield:hover .icons:hover .edit {
  color: blue;
  border-color: blue;
  text-decoration:underline;
  background:url(../../../assets/rapidapp/misc/static/images/bullet_pencil.png) center right no-repeat floralwhite;
}

.appdv-edit-bigfield .icons .save {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_disk.png) center right no-repeat floralwhite;
}

.appdv-edit-bigfield .icons .cancel {
  background:url(../../../assets/rapidapp/misc/static/images/bullet_cross.png) center right no-repeat floralwhite;
}

.appdv-edit-bigfield:hover .icons, 
.appdv-edit-bigfield:hover .icons .edit {
  display:block;
}

.editing .appdv-edit-bigfield .icons .edit, 
.appdv-edit-bigfield:hover .icons .save, 
.appdv-edit-bigfield:hover .icons .cancel {
  display:none;
}

.editing .appdv-edit-bigfield .icons .save, 
.editing .appdv-edit-bigfield .icons .cancel {
  display:block;
  visibility:visible;
  position: relative;
  float:right;
}

.editing .appdv-edit-bigfield .icons .save:hover, 
.editing .appdv-edit-bigfield .icons .cancel:hover {
  color:#B8860B;
}

div.appdv-edit-bigfield .icons .pseudo-edit-wrapper {
  display:none;
  position: absolute;
  top: 0; 
  right: 0;
  bottom: -2px;
  left: -2px;
  border-top-style: none;
  border-right-style: none;
  border-top-width: 0px;
  border-right-width: 0px;
  border-left-style: dotted;
  border-bottom-style: dotted;
  border-left-width: 1px;
  border-bottom-width: 1px;
  border-color: blue;
}

.appdv-edit-bigfield .icons .edit:hover + .pseudo-edit-wrapper {
  display:block !important;
}

/****************** END appdv-edit-bigfield ******************/



/* override to hide edit items if "no_update" is set and delete if no_destroy is set: */
.no_update .edit, .no_destroy .delete-record {
  display: none !important;
}





/*

.appdv-tt-generated .clickable .editable-value .data {
  padding-right: 14px;
}

.appdv-tt-generated .clickable .editable-value .data:hover {
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny.png) center right no-repeat;
  cursor:  pointer;
}

.appdv-tt-generated .clickable .editable-value .data:hover span {
  border-bottom-style: dotted;
  border-bottom-width: 1px;
  border-color: blue;
}

.appdv-tt-generated .clickable .editable-value .save, .appdv-tt-generated .clickable .editable-value .cancel {

  position: absolute;
  top: 0; 
  right: 0;
  padding-top:4px;
  padding-left:4px;
  cursor:  pointer;
  z-index:500;
  font-size: 10px;
  font-family: arial;
  background:url(../../../assets/rapidapp/misc/static/images/pencil_tiny.png) center right no-repeat floralwhite;
  padding:1px 14px 1px 1px;
  white-space:nowrap;
  color: blue;
  font-weight: normal;
  text-decoration:underline;
  border-style: dotted;
  border-width: 1px;
  border-color: blue;
}


.appdv-tt-generated .clickable .editable-value + .editing .save, .appdv-tt-generated .clickable .editable-value + .editing .cancel {
  visibility:visible;
}


*/




/************************************/


/**************************************************************************

  NEW: AppDV inline styles

  The 'inln' class is applied to edit-fields within AppDV server-side code
  at multiple levels in order to make it so that this:

   <span>Foo is [% r.autofield.foo %] said Joe.</span>

  Will render roughly the same as:

   <span>Foo is {foo} said Joe.</span>

  But the former with support inline edit controls          */

div.appdv-tt-generated.inln, 
div.appdv-tt-generated .inln,
div.appdv-tt-generated .appdv-edit-field * {
  display: inline-block;
  max-width:100%;
}

/*  Special block 'blk' override, used for fields like cas-image-field */
/* This right now is about the best we can do, which is to position the
   edit icons to the outside on the right. We set a margin to make room,
   but this margin probably won't be honored depending on the content,
   such is the case of <img> autosized using  with max-width:100%. This
   just means the user markup should leave a little extra gap if the img
   is oversized and they don't want overlap
 */
div.appdv-tt-generated.blk .appdv-edit-field.inln {
  position:relative;
  margin-right:25px;
}

div.appdv-tt-generated.blk .appdv-edit-field.inln .icons {
  position:absolute;
  top:0;
  right:-23px;
  display: inline-block;
}





/**************************************************************************/


div.appdv-tt-generated .editable-value .appdv-edit-field .data-holder {
  line-height: 100%;
  padding-bottom:2px;
}


.appdv-tt-generated .clickable {
  cursor: pointer;
}

.appdv-tt-generated .appdv-edit-bigfield .data-wrapper {
  cursor:  default;
}



.ra-dsbtn-deny-add     .appdv-tt-generated .store-button.add,
.ra-dsapi-deny-create  .appdv-tt-generated .store-button.add,
.ra-dsbtn-deny-edit    .appdv-tt-generated .store-button.edit,
.ra-dsapi-deny-update  .appdv-tt-generated .store-button.edit,
.ra-dsbtn-deny-delete  .appdv-tt-generated .store-button.delete,
.ra-dsapi-deny-destroy .appdv-tt-generated .store-button.delete,
.ra-dsbtn-deny-reload  .appdv-tt-generated .store-button.reload,
.ra-dsbtn-deny-save    .appdv-tt-generated .store-button.save,
.ra-dsbtn-deny-undo    .appdv-tt-generated .store-button.undo {
  opacity: 0.2;
  cursor: default;
}

/* Give enough room for the dirty flag to display 
  (needed to see it with content like images) */
.appdv-tt-generated .x-grid3-dirty-cell {
  padding-left:5px;
}

.appdv-rec-invalids .appdv-tt-generated .edit-record-toggle .save,
.appdv-tt-generated .appdv-field-invalid .appdv-edit-field .icons .save {
  cursor: default;
  pointer-events: none;
  opacity: 0.5;
} 


/* 
  appdv-fill-absolute class name can be set on a tag which encloses a big-field where it uses
  absolute positioning and you want the field and its editor to fill the area completely. This
  is useful for very large text areas that are the last elements of a page.
*/
.appdv-fill-absolute {
  height:100%;
}

.appdv-fill-absolute > .appdv-tt-generated {
  position: relative;
  height: 100%;
  width: 100%;
}

.appdv-fill-absolute > .appdv-tt-generated > .clickable,
.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value
{
  max-height: 100%;
  max-width: 100%;
}

.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield,
.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field {
  position: static;
  max-height: 100%;
  max-width: 100%;
}

.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield .data-holder,
.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield .field-holder .x-form-field,
.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field .data-holder,
.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field .field-holder .x-form-field,
.appdv-fill-absolute > .appdv-tt-generated .appdv-fill-absolute-target-rule {
  position: absolute;
  top: 0px;
  right:0px;
  bottom: 0px;
  left: 0px;
  overflow: auto;
  overflow-x: hidden;
  text-overflow: ellipsis; 
  padding:0px;
  margin: 0px;
}

.appdv-fill-absolute.space-for-icons > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield .data-holder,
.appdv-fill-absolute.space-for-icons > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield .field-holder .x-form-field,
.appdv-fill-absolute.space-for-icons > .appdv-tt-generated .appdv-edit-bigfield .appdv-fill-absolute-target-rule {
  bottom: 17px;
}

.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield .icons {
  position: absolute;
  bottom:0px;
  right:0px;
}

.appdv-fill-absolute.space-for-icons > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field .data-holder,
.appdv-fill-absolute.space-for-icons > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field .field-holder .x-form-field,
.appdv-fill-absolute.space-for-icons > .appdv-tt-generated .appdv-fill-absolute-target-rule {
  right: 38px;
}

.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field .icons {
  position: absolute;
  right:0px;
  bottom:2px;
}

.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-bigfield .icons > *,
.appdv-fill-absolute > .appdv-tt-generated > .clickable > .editable-value > .appdv-edit-field .icons > * {
  float:right;
  position: relative;
}


